package com.malk.xiding.schedule;

import com.alibaba.fastjson.JSON;
import com.malk.Util.UtilDateTime;
import com.malk.Util.UtilMap;
import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.service.aliwork.YDClient;
import com.malk.service.aliwork.YDService;
import com.malk.xiding.server.XDConf;
import com.malk.xiding.service.GSClient;
import com.malk.xiding.service.XDService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import java.time.LocalDate;
import java.util.Date;
import java.util.Map;

/**
 * @EnableScheduling 开启定时任务 [配置参考McScheduleTask]
 */
@Slf4j
@Configuration
@EnableScheduling
@ConditionalOnProperty(name = {"spel.scheduling"})
public class XDScheduleTask {

    @Autowired
    private XDService xdService;

    /**
     * [定时同步审批单]
     * 每天同步[错开高峰]: 流程搜索按照发起和结束时间, 兼容性同步15内审批单
     */
    @Scheduled(cron = "0 0 2,6,23 1/1 * ?")
    public void Time_Period_1() {
        try {
            log.info("###### [XD]同步审批单开始 ######");
            long endTime = new Date().getTime();
            long startTime = endTime - 15 * 24 * 60 * 60 * 1000L;
            xdService.syncApprove(startTime, endTime, XDConf.TMP_CODE_CC, "出差");
            xdService.syncApprove(startTime, endTime, XDConf.TMP_CODE_WC, "外出");
        } catch (Exception e) {
            log.error(e.getMessage(), e); // 记录错误日志
        }
        log.info("###### [XD]同步审批单结束 ######");
    }

    /**
     * [频率为每10分钟查询一次, 同步当天全量时候, 过滤重复考勤, 避免钉钉调用次数过高]
     */
    @Scheduled(cron = "0 9/10 9-10,16-23,0-4 1/1 * ?")
    public void Time_Period_2() {
        log.info("###### [XD]外出同步考勤开始 ######");
        Date now = new Date();
        Date sTime = UtilDateTime.parseDateTime(UtilDateTime.formatDate(now) + " 00:00:00");
        Date eTime = UtilDateTime.parseDateTime(UtilDateTime.formatDate(now) + " 23:59:59");
        try {
            xdService.syncAttendance(0, sTime, eTime);
        } catch (Exception e) {
            log.error(e.getMessage(), e); // 记录错误日志
        }
        log.info("###### [XD]外出同步考勤结束 ######");
    }

    /**
     * prd [2023-07] 外出保留之前纷享销客同步机制, 出差18-22点同步, 自动上传
     */
    @Scheduled(cron = "0 5 18,20,22 * * ?")
    public void Time_Period_3() {
        log.info("###### [XD]出差同步考勤开始 ######");
        Date now = new Date();
        Date sTime = UtilDateTime.parseDateTime(UtilDateTime.formatDate(now) + " 00:00:00");
        Date eTime = UtilDateTime.parseDateTime(UtilDateTime.formatDate(now) + " 23:59:59");
        try {
            xdService.syncAttendanceForCC(0, sTime, eTime);
        } catch (Exception e) {
            log.error(e.getMessage(), e); // 记录错误日志
        }
        log.info("###### [XD]出差同步考勤结束 ######");
    }

    ////////////////////////////// 工时上报 //////////////////////////////

    @Autowired
    private GSClient gsClient;

    @Autowired
    private YDService ydService;

    @Autowired
    private YDClient ydClient;

    /**
     * 每月1号 9.30 推送全员工时填写提醒
     */
    @Scheduled(cron = "0 30 9 1 * ? ")
    public void Time_Period_4() {
        try {
            gsClient.noticeTemplate(false);
        } catch (Exception e) {
            log.error(e.getMessage(), e); // 记录错误日志
        }
    }

    /**
     * 每月6号 9.30 推送月度工时确认审批
     */
    @Scheduled(cron = "0 30 9 6 * ? ")
    public void Time_Period_5() {
        try {
            gsClient.collectMonthReport();
        } catch (Exception e) {
            log.error(e.getMessage(), e); // 记录错误日志
        }
        String ymText = UtilDateTime.formatLocal(LocalDate.now().minusMonths(1L), "yyyy-MM");
        Map condition = UtilMap.map("textField_lnb18vw9", ymText);
        // 查询上月状态
        ydService.upsertFormData("FORM-4UA668D1VTFEFFLLCKRCZ9KH8MGT354H81BNLS1", condition, condition, list -> {
            // 更新确认状态
            return UtilMap.map("radioField_lnb18vwa", "是");
        });
    }

    /**
     * 每月10~31 9.30 推送月度异常工时审批
     */
    @Scheduled(cron = "0 30 9 10-31 * ? ")
    public void Time_Period_6() {
        String ymText = UtilDateTime.formatLocal(LocalDate.now().minusMonths(1L), "yyyy-MM");
        Map condition = UtilMap.map("textField_lnb18vw9", ymText);
        // 查询上月状态
        ydService.upsertFormData("FORM-4UA668D1VTFEFFLLCKRCZ9KH8MGT354H81BNLS1", condition, condition, list -> {
            if (list.size() > 0) {
                Map form = (Map) list.get(0).get("formData");
                // 不执行更新: 月度未确认 | 上报已执行
                if (!"是".equals(form.get("radioField_lnb18vwa")) || "是".equals(form.get("radioField_lnb18vwb"))) {
                    return null;
                }
                boolean isOper = gsClient.exceptionMonthReport();

                // 不执行更新: 月度工时未全部确认 [12.19 异常上报为增量, 当月度全部确认完毕停止任务]
                long count = ydClient.queryData(YDParam.builder()
                        .formUuid("FORM-TD966Z81I9ODTCY66GH345S03VW03JJF6EQLL5")
                        .searchFieldJson(JSON.toJSONString(UtilMap.map("textField_lm5n9tjp", ymText)))
                        .instanceStatus("RUNNING")
                        .build(), YDConf.FORM_QUERY.retrieve_search_process).getTotalCount();
                // 薪资有导入记录, 执行更新状态
                if (isOper && count == 0) {
                    return UtilMap.map("radioField_lnb18vwb", "是");
                }
            }
            // 无记录执行新增
            return condition;
        });
    }

    /**
     * 每月异常上报工时全部处理完成，推送管理员 [每月11号开始, 同时查询有异常上报记录, 且均审批完成]
     */
    @Scheduled(cron = "0 30 9 11-31 * ? ")
    public void Time_Period_7() {
        String ymText = UtilDateTime.formatLocal(LocalDate.now().minusMonths(1L), "yyyy-MM");
        Map condition = UtilMap.map("textField_lnb18vw9", ymText);
        // 查询上月状态
        ydService.upsertFormData("FORM-4UA668D1VTFEFFLLCKRCZ9KH8MGT354H81BNLS1", condition, condition, list -> {
            if (list.size() > 0) {
                Map form = (Map) list.get(0).get("formData");
                // 不执行更新: 异常未上报 | 上报已执行
                if (!"是".equals(form.get("radioField_lnb18vwb")) || "是".equals(form.get("radioField_lnb2j7jd"))) {
                    return null;
                }
                long count = ydClient.queryData(YDParam.builder()
                        .formUuid("FORM-8Y866XB1G4NDRF8RCHWJRB5EQPI13TD3I8ULLT")
                        .searchFieldJson(JSON.toJSONString(UtilMap.map("textField_lm5n9tjp", ymText)))
                        .instanceStatus("RUNNING")
                        .build(), YDConf.FORM_QUERY.retrieve_search_process).getTotalCount();
                // 异常处理结束, 通知管理员; 更新状态
                if (count == 0) {
                    gsClient.noticeTemplate(true);
                    return UtilMap.map("radioField_lnb2j7jd", "是");
                }
            }
            // 无记录执行新增
            return condition;
        });
    }
}